#include "MetaBall.h"
// MetaBall-Animation
// (c) Samuel Kleiser 2011


MetaBall::MetaBall(int w, int h)
{
    step = SCREEN_W / THREADS;
    this->width = w;
    this->height = h;
    bmp = create_bitmap(width, height);
}
MetaBall::~MetaBall()
{
    destroy_bitmap(bmp);
    for(it = ball.begin(); it != ball.end(); it++)
        delete *it;
}
void MetaBall::Update()
{
    BaseUpdate();
    UpdateEx(0, width);
}
void MetaBall::UpdateEx(int start, int end)
{
    for (int x = start; x < end; x++)
        for (int y = 0; y < height; y++)
        {
            double value = 0;
            for(it = ball.begin(); it != ball.end(); it++)
                value += (*it)->w / ((*it)->Distance(x, y) + 0.00001);
            int c = value > 256 ? 0 : (int)(value);
            int color = makecol(c % 256, 2 * c % 256, 5 * c % 256);
            if(buffer[x][y] != color)
            {
                buffer[x][y] = color;
                //_putpixel32(bmp,x,y,color);
            }
        }

}

void MetaBall::BaseUpdate()
{
    for(it = ball.begin(); it != ball.end(); it++)
        for(it2 = ball.begin(); it2 != ball.end(); it2++)
            (*it)->Update(*it2);
    for(it = ball.begin(); it != ball.end(); it++)
        (*it)->DoUpdate(width,height);
}

void MetaBall::UpdateAsync()
{
    BaseUpdate();
    HANDLE hThread[THREADS];
    for(int i = 0; i < THREADS; i++)
    {
        Pack* p = new Pack(this, i);
        hThread[i] = (HANDLE) _beginthreadex(0,0,&UpdateThread,(void*)p,0,0);
    }
    for(int i = 0; i < THREADS; i++)
    {
        WaitForSingleObject( hThread[i], INFINITE );
        CloseHandle( hThread[i] );
    }
}
unsigned __stdcall UpdateThread(void* var)
{
    Pack* p = (Pack*)var;
    p->mb->UpdateEx(p->i * p->mb->step, p->i * p->mb->step + p->mb->step );
    delete p;
    _endthreadex(0);
}
void MetaBall::Draw(BITMAP* dest, int x, int y)
{
    for (int xi = 0; xi < width; xi++)
        for (int yi = 0; yi < height; yi++)
            _putpixel32(dest, x+xi,y+yi,buffer[xi][yi]);
}


Ball::Ball(int x, int y, double dx, double dy, double width, double dw, double brake, double weight)
{
    this->x = x;
    this->y = y;
    this->dx = dx;
    this->dy = dy;
    this->w = width;
    this->dw = dw;
    this->brake = brake;
    this->weight = weight;
}
void Ball::Update(Ball* b)
{
    dx += (b->x - x) * (b->weight / weight) * 0.001;
    dy += (b->y - y) * (b->weight / weight) * 0.001;
}
void Ball::DoUpdate(int width, int height)
{
    x += dx;
    y += dy;
    w += dw;
    dw *= brake;
    if(x < 0)
        dx = ABS(dx);
    if(x > width)
        dx = -ABS(dx);
    if(y < 0)
        dy = ABS(dy);
    if(x > height)
        dy = -ABS(dy);
}
double Ball::Distance(double x, double y)
{
    return QUAD(this->x - x) + QUAD(this->y - y);
}

Pack::Pack(MetaBall* mb, int i)
{
    this->i = i;
    this->mb = mb;
}
